unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  mvMapViewer, mvGpsObj, mvGpx, mvDrawingEngine;

type

  { TMainForm }

  TMainForm = class(TForm)
    ComboBox1: TComboBox;
    MapView: TMapView;
    Panel1: TPanel;
    procedure ComboBox1Change(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure MapViewDrawGpsPoint(Sender: TObject; 
      ADrawer: TMvCustomDrawingEngine; APoint: TGpsPoint);
    procedure MapViewMouseDown(Sender: TObject; Button: TMouseButton; 
      Shift: TShiftState; X, Y: Integer);
  private
    procedure AddLine(ALat1, ALon1, ALat2, ALon2: Double);

  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses
  mvTypes, mvEngine;

const
  _LINE_ = 20;      // Identifies the track that we will be adding (use any number)
  _PTS_ = 21;
  
{ TMainForm }

procedure TMainForm.AddLine(ALat1, ALon1, ALat2, ALon2: Double);
const
  N = 10;     // Number of points along the track
var
  gpsTrack: TGpsTrack;
  lon, lat: Double;
  i: Integer;
begin
  // Create an empty track (it is destroyed by the MapView)
  gpsTrack := TGpsTrack.Create;
  
  // Specify the track's appearance
  gpsTrack.LineColor := clRed;
  gpsTrack.LineWidth := 1;
    
  // Add points to the track from start point (i=0) to end point (i=N-1)
  for i := 0 to N-1 do
  begin
    lat := ALat1 + i*(ALat2 - ALat1) / (N-1);
    lon := ALon1 + i*(ALon2 - ALon1) / (N-1);
    gpsTrack.Points.Add(TGpsPoint.CreateFrom(RealPoint(lat, lon)));
    // The GpsPoints created here will be destroyed by the track.
  end;

  // Add the track to the MapView and identify it by the constant _LINE_
  MapView.GpsItems.Add(gpsTrack, _LINE_);
end;

procedure TMainForm.FormActivate(Sender: TObject);
var
  crs: TCursor;
begin
  crs := Screen.Cursor;
  Screen.Cursor := crHourglass;
  try
    // Threaded painting interferes with track painting over several tiles
    MapView.UseThreads := false;
    MapView.MapProvider := Combobox1.Text;
    MapView.Active := true;
    MapView.Center := RealPoint(40.0, 10.0);
    MapView.Zoom := 3;
    
    AddLine(3.0, -60.0, 65.0, 75.0);
  finally
    Screen.Cursor := crs;
  end;
end;

procedure TMainForm.MapViewDrawGpsPoint(Sender: TObject; 
  ADrawer: TMvCustomDrawingEngine; APoint: TGpsPoint);
const
  R = 8;
var
  p: TPoint;
  w: Integer;
begin
  // Screen coordinates of the GPS point
  p := MapView.LonLatToScreen(APoint.RealPoint);
 
  // Draw a blue circle
  ADrawer.PenColor := clBlack;
  ADrawer.BrushColor := clBlue;
  ADrawer.BrushStyle := bsSolid;
  ADrawer.Ellipse(p.X-R, p.Y-R, P.X+R, p.Y+R);
 
  // Draw the point label
  ADrawer.BrushStyle := bsClear;
  ADrawer.FontName := 'SegoeUI';
  ADrawer.FontSize := 16;
  ADrawer.FontStyle := [fsBold];
  w := ADrawer.TextWidth(APoint.Name);
  ADrawer.TextOut(
    p.X - ADrawer.TextWidth(APoint.Name) div 2,
    p.Y + R + 4,
    APoint.Name
  );
end;

procedure TMainForm.MapViewMouseDown(Sender: TObject; Button: TMouseButton; 
  Shift: TShiftState; X, Y: Integer);
var
  rp: TRealPoint;
  P: TGpsPoint;
begin
  if Button = mbRight then
  begin
    rp := MapView.ScreenToLonLat(Point(X, Y));
    P := TGpsPoint.CreateFrom(rp);
    P.Name := Format('%s' + LineEnding + '%s', [LatToStr(rp.Lat, true), LonToStr(rp.Lon, true)]);;
    MapView.GpsItems.Add(P, _PTS_);
  end;
end;

procedure TMainForm.ComboBox1Change(Sender: TObject);
begin
  MapView.MapProvider := Combobox1.Text;
end;

end.

